from blockerbugs.models.bug import Bug
from blockerbugs.models.release import Release
from blockerbugs.util import bz_interface
from datetime import datetime
from sqlalchemy import or_
import re
import logging


class BugSync():
    def __init__(self, db, bzinterface=None):
        self.db = db
        self.closedre = re.compile('.*CLOSED.*')
        if not bzinterface:
            self.bzinterface = bz_interface.BlockerBugs()
        else:
            self.bzinterface = bzinterface
        self.log = logging.getLogger('bug_sync')

    def extract_information(self, bug, tracker_type):
        buginfo = {}
        bug_whiteboard = bug.whiteboard

        # get information about accepted/rejected
        buginfo['accepted'] = ('Accepted%s' % tracker_type) in bug_whiteboard
        buginfo['rejected'] = ('Rejected%s' % tracker_type) in bug_whiteboard
        buginfo['proposed'] = not (buginfo['rejected'] or buginfo['accepted'])

        # determine whether or not the bug is active (not CLOSED)
        buginfo['active'] = not self.closedre.match(bug.status)

        # basic info
        buginfo['bug_id'] = bug.bug_id
        buginfo['component'] = bug.component
        buginfo['summary'] = bug.summary
        buginfo['status'] = bug.status
        buginfo['url'] = bug.url
        buginfo['last_change_time'] = datetime.strptime(bug.delta_ts, '%Y-%m-%d %H:%M:%S')

        return buginfo

    def get_tracker_id(self, release, tracker_type):
        if tracker_type == 'Blocker':
            return release.blocker_tracker
        elif tracker_type == 'NTH':
            return release.accepted_tracker

    def update_release(self, release, tracker_type, lastupdate=None):
        trackerid = self.get_tracker_id(release, tracker_type)

        self.log.info('Updating %s for Fedora %d %s (%d)' % (tracker_type,
                                                release.number,
                                                release.version,
                                                trackerid))

        newbugs = self.bzinterface.do_query(trackerid, last_update=lastupdate)
        self.log.debug('New bugs: %s' % str(newbugs))
        for newbug in newbugs:
            oldbug = Bug.query.filter_by(bugid=newbug.bug_id, release=release).first()
            newbuginfo = self.extract_information(newbug, tracker_type)
            if oldbug:
                self.log.info('Updating bug %d' % newbuginfo['bug_id'])
                oldbug.update(newbuginfo, tracker_type, release)
            else:
                self.log.info('Adding new bug %d' % newbuginfo['bug_id'])
                oldbug = Bug.from_data(newbuginfo, release, tracker_type)

            self.db.session.add(oldbug)
            self.db.session.commit()


    def cleanup_release(self, release, tracker_type):
        trackerid = self.get_tracker_id(release, tracker_type)

        self.log.info('Cleaning up  %s for Fedora %d %s (%d)' % (tracker_type,
                                                release.number,
                                                release.version,
                                                trackerid))

        tracker_depends = self.bzinterface.get_deps(trackerid)
        self.log.info("Current dependencies for %d: %s" % (trackerid, str(tracker_depends)))
        if tracker_type == 'NTH':
            blockers = Bug.query.filter_by(release=release).filter(or_(Bug.proposed_nth == True,
                                                                    Bug.accepted_nth == True)).all()
        elif tracker_type == 'Blocker':
            blockers = Bug.query.filter_by(release=release).filter(or_(Bug.proposed_blocker == True,
                                                                    Bug.accepted_blocker == True)).all()
        for blocker in blockers:
            if not blocker.bugid in tracker_depends:
                if tracker_type == 'NTH':
                    blocker.proposed_nth = False
                    blocker.accepted_nth = False
                    blocker.rejected_nth = True
                elif tracker_type == 'Blocker':
                    blocker.proposed_blocker = False
                    blocker.accepted_blocker = False
                    blocker.rejected_blocker = True
                self.db.session.add(blocker)
            self.db.session.commit()


    def update_active(self, lastupdate=None):
        starttime = datetime.utcnow()
        active_releases = Release.query.filter_by(active=True).all()

        # for each active release, update the tracker to remove any bugs that
        # no longer block tracker and pull in any new bugs
        for release in active_releases:
            self.cleanup_release(release, 'Blocker')
            self.update_release(release, 'Blocker', release.last_updated)

            self.cleanup_release(release, 'NTH')
            self.update_release(release, 'NTH', release.last_updated)

            # update the last query time for the release
            release.last_updated = starttime
            self.db.session.add(release)
            self.db.session.commit()


