#
# bz_interface.py - common methods for obtaining blocker/nth information from
#                   RHBZ
#
# Copyright 2012, Red Hat, Inc
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# Authors:
#   Tim Flink <tflink@redhat.com>


import bugzilla
import logging

BUG_STATUS = ['NEW', 'ASSIGNED', 'ON_DEV', 'MODIFIED', 'POST', 'ON_QA', 'FAILS_QA',
              'PASSES_QA', 'REOPENED', 'VERIFIED', 'RELEASE_PENDING']

base_query = {'status': BUG_STATUS,
            'o1': 'anywords',
            'f1': 'blocked',
            'query_format':'advanced'}


class BlockerBugs():
    def __init__(self, bz=None):
        if not bz:
            self.bz = bugzilla.RHBugzilla4(url='https://bugzilla.redhat.com/xmlrpc.cgi')
        else:
            self.bz = bz
        self.log = logging.getLogger('bz_interface')

    def get_bz_query(self, tracker, last_update=None):
        query = {}
        query.update(base_query)
        query['v1'] = str(tracker)
        if last_update:
            last_update_string = last_update.strftime("%Y-%m-%d %H:%M GMT")

            # Since this is kind of confusing, the idea here is to only grab only
            # the bugs whose whiteboard, blocks or dependson has changed since
            # the last sync.
            # keep in mind that OP -> open parenthesis, CP -> closed parenthesis
            # and the OR is to make sure that a bug is returned if ANY of the
            # changed after conditions are met
            query.update({
               'f2': 'OP',
               'j2': 'OR',
               'f3': 'blocked',
               'v3': last_update_string,
               'o3': 'changedafter',
               'f4': 'status_whiteboard',
               'v4': last_update_string,
               'o4': 'changedafter',
               'f5': 'bug_status',
               'v5': last_update_string,
               'o5': 'changedafter',
               'f6': 'dependson',
               'v6': last_update_string,
               'o6': 'changedafter',
               'f7': 'CP'
            })
        return query


    def do_query(self, tracker, last_update=None):
        query = self.get_bz_query(tracker, last_update)
        buglist = self.flatten_bug_list(query)
        self.log.debug('Doing query: %s' % str(query))
        return buglist


    # instead of having a bunch of nested bug objects, make a final list of
    # all of the applicable bugs
    def flatten_bug_list(self, query, deps=None, max_recurse=3):
        result = self.bz.query(query)

        if max_recurse <= 0:
            return result

        if deps == None:
            deps = []

        for i in range(len(result)):
            b = result[i]
            if len(b.dependson) > 0:
                for dep in b.dependson:
                    if dep not in deps:
                        deps.append(dep)
                        result.extend(self.flatten_bug_list(self.get_bz_query(dep),
                                        max_recurse=max_recurse-1, deps=deps))

        return result

    def get_deps(self, bugid):
        return self.bz.getbug(bugid).dependson

